@@ -9,15 +9,10 @@ inline json_char ascii_one(void) json_nothrow {
 #ifdef JSON_READ_PRIORITY
 
 JSONNode JSONWorker::parse(const json_string & json) json_throws(std::invalid_argument) {
-    json_auto<json_char> s;
-    #if defined JSON_DEBUG || defined JSON_SAFE
-	   json_char lastchar;
-	   s.set(RemoveWhiteSpace(json, lastchar, true));
-	   return _parse_unformatted(s.ptr, lastchar);
-    #else
-	   s.set(RemoveWhiteSpace(json, true));
-	   return _parse_unformatted(s.ptr);
-    #endif
+	json_auto<json_char> s;
+	size_t len;
+	s.set(RemoveWhiteSpace(json, len, true));
+	return _parse_unformatted(s.ptr, s.ptr + len);
 }
 
 JSONNode JSONWorker::parse_unformatted(const json_string & json) json_throws(std::invalid_argument) {
@@ -27,18 +22,11 @@ JSONNode JSONWorker::parse_unformatted(const json_string & json) json_throws(std
 	   #else
 		  JSON_ASSERT_SAFE((json[0] == JSON_TEXT('{')) || (json[0] == JSON_TEXT('[')), JSON_TEXT("Not JSON!"), return JSONNode(JSON_NULL););
 	   #endif
-	   json_char temp = (json[0] == JSON_TEXT('{')) ? JSON_TEXT('}') : JSON_TEXT(']');
-	   return _parse_unformatted(json.c_str(), temp);
-    #else
-	   return _parse_unformatted(json.c_str());
     #endif
+	return _parse_unformatted(json.data(), json.data() + json.length());
 }
 
-#if defined JSON_DEBUG || defined JSON_SAFE
-JSONNode JSONWorker::_parse_unformatted(const json_char * json, json_char & lastchar) json_throws(std::invalid_argument) {
-#else
-JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std::invalid_argument) {
-#endif
+JSONNode JSONWorker::_parse_unformatted(const json_char * json, const json_char * const end) json_throws(std::invalid_argument) {
     #ifdef JSON_COMMENTS
 	   json_char firstchar = *json;
 	   json_string _comment;
@@ -46,7 +34,7 @@ JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std:
 	   if (json_unlikely(firstchar == JSON_TEMP_COMMENT_IDENTIFIER)){  //multiple comments will be consolidated into one
 		  newcomment:
 		  while(*(++runner) != JSON_TEMP_COMMENT_IDENTIFIER){
-			 JSON_ASSERT(*runner, JSON_TEXT("Removing white space failed"));
+			 JSON_ASSERT(runner != end, JSON_TEXT("Removing white space failed"));
 			 _comment += *runner;
 		  }
 		  firstchar = *(++runner); //step past the trailing tag
@@ -64,23 +52,23 @@ JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std:
         case JSON_TEXT('['):
 		  #if defined JSON_DEBUG || defined JSON_SAFE
 			 if (firstchar == JSON_TEXT('[')){
-				if (json_unlikely(lastchar != JSON_TEXT(']'))){
+				if (json_unlikely(*(end - 1) != JSON_TEXT(']'))){
 				    JSON_FAIL(JSON_TEXT("Missing final ]"));
 				    break;
 				}
 			 } else {
-				if (json_unlikely(lastchar != JSON_TEXT('}'))){
+				if (json_unlikely(*(end - 1) != JSON_TEXT('}'))){
 				    JSON_FAIL(JSON_TEXT("Missing final }"));
 				    break;
 				}
 			 }
 		  #endif
 		  #ifdef JSON_COMMENTS
-			 JSONNode foo(runner);
+			 JSONNode foo(json_string(runner, end - runner));
 			 foo.set_comment(_comment);
 			 return JSONNode(true, foo);  //forces it to simply return the original interal, even with ref counting off
 		  #else
-			 return JSONNode(json);
+			 return JSONNode(json_string(json, end - json));
 		  #endif
     }
 
@@ -130,10 +118,18 @@ JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std:
 
 
 
-#ifdef JSON_READ_PRIORITY
-    size_t JSONWorker::FindNextRelevant(json_char ch, const json_string & value_t, const size_t pos) json_nothrow {
-	   const json_char * start = value_t.c_str();
-	   for (const json_char * p = start + pos; *p; ++p){
+#if defined(JSON_READ_PRIORITY) || defined(JSON_STREAM)
+	#if (JSON_READ_PRIORITY == HIGH) && (!(defined(JSON_LESS_MEMORY)))
+		#define FIND_NEXT_RELEVANT(ch, vt, po) JSONWorker::FindNextRelevant<ch>(vt, po)
+		template<json_char ch>
+		size_t JSONWorker::FindNextRelevant(const json_string & value_t, const size_t pos) json_nothrow {
+	#else
+		#define FIND_NEXT_RELEVANT(ch, vt, po) JSONWorker::FindNextRelevant(ch, vt, po)
+		size_t JSONWorker::FindNextRelevant(json_char ch, const json_string & value_t, const size_t pos) json_nothrow {
+	#endif
+		json_string::const_iterator start = value_t.begin();
+		json_string::const_iterator e = value_t.end();
+	   for (json_string::const_iterator p = value_t.begin() + pos; p != e; ++p){
 		  if (json_unlikely(*p == ch)) return p - start;
 		  switch (*p){
 				BRACKET(JSON_TEXT('['), JSON_TEXT(']'))
@@ -148,10 +144,10 @@ JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std:
 #ifdef JSON_COMMENTS
     #define COMMENT_DELIMITER() *runner++ = JSON_TEMP_COMMENT_IDENTIFIER
     #define AND_RUNNER ,runner
-    inline void SingleLineComment(const json_char * & p, json_char * & runner) json_nothrow {
+    inline void SingleLineComment(const json_char * & p, const json_char * const end, json_char * & runner) json_nothrow {
 	   //It is okay to add two '\5' characters here because at minimun the # and '\n' are replaced, so it's at most the same size
 	   COMMENT_DELIMITER();
-	   while((*(++p)) && (*p != JSON_TEXT('\n'))){
+	   while((++p != end) && (*p != JSON_TEXT('\n'))){
 		  *runner++ = *p;
 	   }
 	   COMMENT_DELIMITER();
@@ -162,136 +158,106 @@ JSONNode JSONWorker::_parse_unformatted(const json_char * json) json_throws(std:
 #endif
 
 #ifndef JSON_STRICT
-inline void SingleLineComment(const json_char * & p) json_nothrow {
-    while((*(++p)) && (*p != JSON_TEXT('\n')));
+inline void SingleLineComment(const json_char * & p, const json_char * const end) json_nothrow {
+    while((++p != end) && (*p != JSON_TEXT('\n')));
 }
 #endif
 
-#ifdef JSON_READ_PRIORITY
-#if defined JSON_DEBUG || defined JSON_SAFE
-    json_char * JSONWorker::RemoveWhiteSpace(const json_string & value_t, json_char & last, bool escapeQuotes) json_nothrow  {
+#if defined(JSON_LESS_MEMORY) && defined(JSON_READ_PRIORITY)
+	#define PRIVATE_REMOVEWHITESPACE(T, value_t, escapeQuotes, len) private_RemoveWhiteSpace(T, value_t, escapeQuotes, len)
+	json_char * private_RemoveWhiteSpace(bool T, const json_string & value_t, bool escapeQuotes, size_t & len) json_nothrow {
 #else
-    json_char * JSONWorker::RemoveWhiteSpace(const json_string & value_t, bool escapeQuotes) json_nothrow {
+	#define PRIVATE_REMOVEWHITESPACE(T, value_t, escapeQuotes, len) private_RemoveWhiteSpace<T>(value_t, escapeQuotes, len)
+	template<bool T>
+	json_char * private_RemoveWhiteSpace(const json_string & value_t, bool escapeQuotes, size_t & len) json_nothrow {
 #endif
-	   json_char * result;
-	   json_char * runner = result = json_malloc<json_char>(value_t.length() + 1);  //dealing with raw memory is faster than adding to a json_string
-	   JSON_ASSERT(result != 0, json_global(ERROR_OUT_OF_MEMORY));
-	   #ifdef JSON_SAFE
-	   try {
-	   #endif
-		   for(const json_char * p = value_t.c_str(); *p; ++p){
-			  switch(*p){
-				 case JSON_TEXT(' '):   //defined as white space
-				 case JSON_TEXT('\t'):  //defined as white space
-				 case JSON_TEXT('\n'):  //defined as white space
-				 case JSON_TEXT('\r'):  //defined as white space
-					break;
-				 #ifndef JSON_STRICT
-					case JSON_TEXT('/'):  //a C comment
-						if (*(++p) == JSON_TEXT('*')){  //a multiline comment
-						   COMMENT_DELIMITER();
-						   while ((*(++p) != JSON_TEXT('*')) || (*(p + 1) != JSON_TEXT('/'))){
-							  JSON_ASSERT_SAFE(*p, JSON_TEXT("Null terminator inside of a multiline quote"), COMMENT_DELIMITER(); throw false;);
-							  *runner++ = *p;
-						   }
-						   ++p;
-						   COMMENT_DELIMITER();
-						   break;
+	json_char * result;
+	json_char * runner = result = json_malloc<json_char>(value_t.length() + 1);  //dealing with raw memory is faster than adding to a json_string
+	JSON_ASSERT(result != 0, json_global(ERROR_OUT_OF_MEMORY));
+	const json_char * const end = value_t.data() + value_t.length();
+	for(const json_char * p = value_t.data(); p != end; ++p){
+	  switch(*p){
+		 case JSON_TEXT(' '):   //defined as white space
+		 case JSON_TEXT('\t'):  //defined as white space
+		 case JSON_TEXT('\n'):  //defined as white space
+		 case JSON_TEXT('\r'):  //defined as white space
+			break;
+		#ifndef JSON_STRICT
+			case JSON_TEXT('/'):  //a C comment
+				if (*(++p) == JSON_TEXT('*')){  //a multiline comment
+				   if (T) COMMENT_DELIMITER();
+				   while ((*(++p) != JSON_TEXT('*')) || (*(p + 1) != JSON_TEXT('/'))){
+					  if(p == end){
+							COMMENT_DELIMITER(); 
+							goto endofrunner;
 						}
-						//Should be a single line C comment, so let it fall through to use the bash comment stripper
-						JSON_ASSERT_SAFE(*p == JSON_TEXT('/'), JSON_TEXT("stray / character, not quoted, or a comment"), throw false;);
-					case JSON_TEXT('#'):  //a bash comment
-						SingleLineComment(p AND_RUNNER);
-						break;
-				 #endif
-				 case JSON_TEXT('\"'):  //a quote
-					*runner++ = JSON_TEXT('\"');
-					while(*(++p) != JSON_TEXT('\"')){  //find the end of the quotation, as white space is preserved within it
-						JSON_ASSERT_SAFE(*p, JSON_TEXT("Null terminator inside of a quotation"), throw false;);
-						switch(*p){
-						   case JSON_TEXT('\\'):
-							  *runner++ = JSON_TEXT('\\');
-                              if (escapeQuotes){
-                                    *runner++ = (*++p == JSON_TEXT('\"')) ? ascii_one() : *p;  //an escaped quote will reak havoc will all of my searching functions, so change it into an illegal character in JSON for convertion later on
-                              }
-							  break;
-						   default:
-							  *runner++ = *p;
-							  break;
-						}
-					}
-					//no break, let it fall through so that the trailing quote gets added
-				 default:
-					JSON_ASSERT_SAFE((json_uchar)*p >= 32, JSON_TEXT("Invalid JSON character detected (lo)"), throw false;);
-					JSON_ASSERT_SAFE((json_uchar)*p <= 126, JSON_TEXT("Invalid JSON character detected (hi)"), throw false;);
-					*runner++ = *p;
-					break;
-			  }
-		   }
-	   #ifdef JSON_SAFE
-	   } catch(...){}
-	   #endif
-	   #if defined JSON_DEBUG || defined JSON_SAFE
-		  last = *(runner - 1);
-	   #endif
-	   *runner = JSON_TEXT('\0');
-	   return result;
+					  if (T) *runner++ = *p;
+				   }
+				   ++p;
+				   if (T) COMMENT_DELIMITER();
+				   break;
+				}
+				//Should be a single line C comment, so let it fall through to use the bash comment stripper
+				JSON_ASSERT_SAFE(*p == JSON_TEXT('/'), JSON_TEXT("stray / character, not quoted, or a comment"), goto endofrunner;);
+			case JSON_TEXT('#'):  //a bash comment
+				if (T){
+					SingleLineComment(p, end AND_RUNNER);
+				} else {
+					SingleLineComment(p, end);
+				}
+				break;
+		 #endif
+		 case JSON_TEXT('\"'):  //a quote
+			*runner++ = JSON_TEXT('\"');
+			while(*(++p) != JSON_TEXT('\"')){  //find the end of the quotation, as white space is preserved within it
+				if(p == end) goto endofrunner;
+				switch(*p){
+				   case JSON_TEXT('\\'):
+					  *runner++ = JSON_TEXT('\\');
+					  if (escapeQuotes){
+							*runner++ = (*++p == JSON_TEXT('\"')) ? ascii_one() : *p;  //an escaped quote will reak havoc will all of my searching functions, so change it into an illegal character in JSON for convertion later on
+					  } else {
+							*runner++ = *++p;
+					  }
+					  break;
+				   default:
+					  *runner++ = *p;
+					  break;
+				}
+			}
+			//no break, let it fall through so that the trailing quote gets added
+		 default:
+			JSON_ASSERT_SAFE((json_uchar)*p >= 32, JSON_TEXT("Invalid JSON character detected (lo)"), goto endofrunner;);
+			JSON_ASSERT_SAFE((json_uchar)*p <= 126, JSON_TEXT("Invalid JSON character detected (hi)"), goto endofrunner;);
+			*runner++ = *p;
+			break;
+	  }
+	}
+	endofrunner:
+	len = runner - result;
+	return result;
+}
+
+#ifdef JSON_READ_PRIORITY
+    json_char * JSONWorker::RemoveWhiteSpace(const json_string & value_t, size_t & len, bool escapeQuotes) json_nothrow  {
+		json_char * result = PRIVATE_REMOVEWHITESPACE(true, value_t, escapeQuotes, len); 
+		result[len] = JSON_TEXT('\0');
+		return result;
     }
 #endif
 
+json_char * JSONWorker::RemoveWhiteSpaceAndCommentsC(const json_string & value_t, bool escapeQuotes) json_nothrow {
+	size_t len;
+	json_char * result = PRIVATE_REMOVEWHITESPACE(false, value_t, escapeQuotes, len);
+	result[len] = JSON_TEXT('\0');
+	return result;
+}
+
 json_string JSONWorker::RemoveWhiteSpaceAndComments(const json_string & value_t, bool escapeQuotes) json_nothrow {
-    json_string result;
-    result.reserve(value_t.length());
-	try {
-		for(const json_char * p = value_t.c_str(); *p; ++p){
-		   switch(*p){
-			  case JSON_TEXT(' '):   //defined as white space
-			  case JSON_TEXT('\t'):  //defined as white space
-			  case JSON_TEXT('\n'):  //defined as white space
-			  case JSON_TEXT('\r'):  //defined as white space
-				 break;
-			  #ifndef JSON_STRICT
-				 case JSON_TEXT('/'):  //a C comment
-					if (*(++p) == JSON_TEXT('*')){  //a multiline comment
-						while ((*(++p) != JSON_TEXT('*')) || (*(p + 1) != JSON_TEXT('/'))){
-						   if (json_unlikely(*p == JSON_TEXT('\0'))) throw false;
-						}
-						++p;
-						break;
-					}
-					//Should be a single line C comment, so let it fall through to use the bash comment stripper
-					JSON_ASSERT_SAFE(*p == JSON_TEXT('/'), JSON_TEXT("stray / character, not quoted, or a comment"), throw false;);
-				 case JSON_TEXT('#'):  //a bash comment
-					SingleLineComment(p);
-					break;
-			  #endif
-			  case JSON_TEXT('\"'):  //a quote
-				 result += JSON_TEXT('\"');
-				 while(*(++p) != JSON_TEXT('\"')){  //find the end of the quotation, as white space is preserved within it
-					switch(*p){
-						case JSON_TEXT('\\'):
-						   result += JSON_TEXT('\\');
-                            if (escapeQuotes){
-                                result += (*++p == JSON_TEXT('\"')) ? ascii_one() : *p;  //an escaped quote will reak havoc will all of my searching functions, so change it into an illegal character in JSON for convertion later on
-                            }
-						   break;
-						case '\0':
-							throw false;
-						default:
-						   result += *p;
-						   break;
-					}
-				 }
-				 //no break, let it fall through so that the trailing quote gets added
-			  default:
-				 JSON_ASSERT_SAFE((json_uchar)*p >= 32, JSON_TEXT("Invalid JSON character detected (lo)"), throw false;);
-				 JSON_ASSERT_SAFE((json_uchar)*p <= 126, JSON_TEXT("Invalid JSON character detected (hi)"), throw false;);
-				 result += *p;
-				 break;
-		   }
-		}
-	} catch (...){}
-    return result;
+	json_auto<json_char> s;
+    size_t len;
+	s.set(PRIVATE_REMOVEWHITESPACE(false, value_t, escapeQuotes, len)); 
+	return json_string(s.ptr, len);
 }
 
 #ifdef JSON_READ_PRIORITY
@@ -307,14 +273,14 @@ json_string JSONWorker::RemoveWhiteSpaceAndComments(const json_string & value_t,
 	   return (((hi << 10) & 0x1FFC00) + 0x10000) | lo & 0x3FF;
     }
 
-    json_string JSONWorker::UTF(const json_char * & pos) json_nothrow {
-	   json_string result;
-	   json_uchar first = UTF8(pos);
+    void JSONWorker::UTF(const json_char * & pos, json_string & result, const json_char * const end) json_nothrow {
+		JSON_ASSERT_SAFE(((intptr_t)end - (intptr_t)pos) > 4, JSON_TEXT("UTF will go out of bounds"), return;);
+	   json_uchar first = UTF8(pos, end);
 	   if (json_unlikely((first > 0xD800) && (first < 0xDBFF) &&
 		  (*(pos + 1) == '\\') && (*(pos + 2) == 'u'))){
 			 const json_char * original_pos = pos;  //if the 2nd character is not correct I need to roll back the iterator
 			 pos += 2;
-			 json_uchar second = UTF8(pos);
+			 json_uchar second = UTF8(pos, end);
 			 //surrogate pair, not two characters
 			 if (json_unlikely((second > 0xDC00) && (second < 0xDFFF))){
 				result += SurrogatePair(first, second);
@@ -324,12 +290,11 @@ json_string JSONWorker::RemoveWhiteSpaceAndComments(const json_string & value_t,
 	   } else {
 		  result += first;
 	   }
-	   JSON_ASSERT(!result.empty(), JSON_TEXT("missing case, somehow UTF returned empty"));
-	   return result;
     }
 #endif
 
-json_uchar JSONWorker::UTF8(const json_char * & pos) json_nothrow {
+json_uchar JSONWorker::UTF8(const json_char * & pos, const json_char * const end) json_nothrow {
+	JSON_ASSERT_SAFE(((intptr_t)end - (intptr_t)pos) > 4, JSON_TEXT("UTF will go out of bounds"), return JSON_TEXT('\0'););
     #ifdef JSON_UNICODE
 	   ++pos;
 	   json_uchar temp = Hex(pos) << 8;
@@ -371,14 +336,15 @@ json_char JSONWorker::Hex(const json_char * & pos) json_nothrow {
 }
 
 #ifndef JSON_STRICT
-    inline json_char FromOctal(const json_char * & str) json_nothrow {
-	   JSON_ASSERT(json_strlen(str) > 3, JSON_TEXT("Octal will go out of bounds"));
+    inline json_char FromOctal(const json_char * & str, const json_char * const end) json_nothrow {
+	   JSON_ASSERT_SAFE(((intptr_t)end - (intptr_t)str) > 3, JSON_TEXT("Octal will go out of bounds"), return JSON_TEXT('\0'););
 	   str += 2;
 	   return (json_char)(((((json_uchar)(*(str - 2) - 48))) << 6) | (((json_uchar)(*(str - 1) - 48)) << 3) | ((json_uchar)(*str - 48)));
     }
 #endif
 
-void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_nothrow {
+void JSONWorker::SpecialChar(const json_char * & pos, const json_char * const end, json_string & res) json_nothrow {
+	JSON_ASSERT_SAFE(pos != end, JSON_TEXT("Special char termantion"), return;);
     /*
 	   Since JSON uses forward slash escaping for special characters within strings, I have to
 	   convert these escaped characters into C characters
@@ -413,13 +379,14 @@ void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_no
 		  break;
 	   case JSON_TEXT('u'):	//utf character
 		  #ifdef JSON_UNICODE
-			 res += UTF(pos);
+			 UTF(pos, res, end);
 		  #else
-			 res += UTF8(pos);
+			 res += UTF8(pos, end);
 		  #endif
 		  break;
 	   #ifndef JSON_STRICT
 		  case JSON_TEXT('x'):   //hexidecimal ascii code
+			 JSON_ASSERT_SAFE(((intptr_t)end - (intptr_t)pos) > 3, JSON_TEXT("Hex will go out of bounds"), res += JSON_TEXT('\0'); return;);
 			 res += Hex(++pos);
 			 break;
 
@@ -436,7 +403,7 @@ void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_no
 			 case JSON_TEXT('6'):
 			 case JSON_TEXT('7'):
 		  #endif
-			 res += FromOctal(pos);
+			 res += FromOctal(pos, end);
 			 break;
 		  default:
 			 res += *pos;
@@ -469,18 +436,20 @@ void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_no
     setflag(false);
     json_string res;
     res.reserve(value_t.length());	 //since it goes one character at a time, want to reserve it first so that it doens't have to reallocating
-    for(const json_char * p = value_t.c_str(); *p; ++p){
+	const json_char * const end = value_t.data() + value_t.length();
+    for(const json_char * p = value_t.data(); p != end; ++p){
 	   switch (*p){
 		  case JSON_TEXT('\\'):
 			 setflag(true);
-			 SpecialChar(++p, res);
+			 SpecialChar(++p, end, res);
 			 break;
 		  default:
 			 res += *p;
 			 break;
 	   }
     }
-    return shrinkString(res);  //because this is actually setting something to be stored, shrink it it need be
+	shrinkString(res);  //because this is actually setting something to be stored, shrink it it need be
+	return res;
 }
 #endif
 
@@ -495,7 +464,7 @@ void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_no
 		  unsigned short HiSurrogate = 0xD800 | (((unsigned short)((unsigned int)((C >> 16) & 31)) - 1) << 6) | ((unsigned short)C) >> 10;
 
 		  //compute the low surrogate
-		  unsigned short LoSurrogate = (unsigned short) (0xDC00 | ((unsigned short)C) & 1023);
+		  unsigned short LoSurrogate = (unsigned short) (0xDC00 | ((unsigned short)C & 1023));
 
 		  json_string res;
 		  res += toUTF8(HiSurrogate);
@@ -535,12 +504,14 @@ void JSONWorker::SpecialChar(const json_char * & pos, json_string & res) json_no
     }
 #endif
 
-json_string JSONWorker::UnfixString(const json_string & value_t, bool flag) json_nothrow {
-    if (!flag) return value_t;
+void JSONWorker::UnfixString(const json_string & value_t, bool flag, json_string & res) json_nothrow {
+    if (!flag){
+		res += value_t;
+		return;
+	}
     //Re-escapes a json_string so that it can be written out into a JSON file
-    json_string res;
-    res.reserve(value_t.length());  //since it goes one character at a time, want to reserve it first so that it doens't have to reallocating
-    for(const json_char * p = value_t.c_str(); *p; ++p){
+	const json_char * const end = value_t.data() + value_t.length();
+    for(const json_char * p = value_t.data(); p != end; ++p){
 	   switch(*p){
 		  case JSON_TEXT('\"'):  //quote character
 			 res += JSON_TEXT("\\\"");
@@ -568,11 +539,13 @@ json_string JSONWorker::UnfixString(const json_string & value_t, bool flag) json
 				res += JSON_TEXT("\\f");
 				break;
 			 default:
+			 {
 				if (json_unlikely(((json_uchar)(*p) < 32) || ((json_uchar)(*p) > 126))){
 				    res += toUTF8((json_uchar)(*p));
 				} else {
 				    res += *p;
 				}
+			}
 				break;
 		  #else
 			 default:
@@ -581,7 +554,6 @@ json_string JSONWorker::UnfixString(const json_string & value_t, bool flag) json
 		  #endif
 	   }
     }
-    return res;
 }
 
 #ifdef JSON_READ_PRIORITY
@@ -597,15 +569,23 @@ inline void JSONWorker::NewNode(const internalJSONNode * parent, const json_stri
 	   START_MEM_SCOPE
 		  json_string _comment;
 		  START_MEM_SCOPE
-			 const json_char * runner = ((array) ? value.c_str() : name.c_str());
+			 const json_char * runner = ((array) ? value.data() : name.data());
+			 #ifdef JSON_DEBUG
+				const json_char * const end = runner + value.length();
+			#endif
 			 if (json_unlikely(*runner == JSON_TEMP_COMMENT_IDENTIFIER)){  //multiple comments will be consolidated into one
-				newcomment:
+				size_t count;
+				const json_char * start;
+			    newcomment:
+				count = 0;
+				start = runner + 1;
 				while(*(++runner) != JSON_TEMP_COMMENT_IDENTIFIER){
-				    JSON_ASSERT(*runner, JSON_TEXT("Removing white space failed"));
-				    _comment += *runner;
+				    JSON_ASSERT(runner != end, JSON_TEXT("Removing white space failed"));
+					++count;
 				}
+				if (count) _comment += json_string(start, count);
 				if (json_unlikely(*(++runner) == JSON_TEMP_COMMENT_IDENTIFIER)){ //step past the trailing tag
-				    _comment += '\n';
+				    _comment += JSON_TEXT('\n');
 				    goto newcomment;
 				}
 			 }
@@ -622,73 +602,73 @@ inline void JSONWorker::NewNode(const internalJSONNode * parent, const json_stri
 	   const_cast<internalJSONNode*>(parent) -> CHILDREN -> push_back(child);   //attach it to the parent node
     #else
 	if (name.empty()){
-	   	const_cast<internalJSONNode*>(parent) -> CHILDREN -> push_back(JSONNode::newJSONNode(internalJSONNode::newInternal(name.c_str(), value)));	    //attach it to the parent node
+	   	const_cast<internalJSONNode*>(parent) -> CHILDREN -> push_back(JSONNode::newJSONNode(internalJSONNode::newInternal(name, value)));	    //attach it to the parent node
 	} else {
-		const_cast<internalJSONNode*>(parent) -> CHILDREN -> push_back(JSONNode::newJSONNode(internalJSONNode::newInternal(name.c_str() + 1, value)));	    //attach it to the parent node
+		const_cast<internalJSONNode*>(parent) -> CHILDREN -> push_back(JSONNode::newJSONNode(internalJSONNode::newInternal(json_string(name.begin() + 1, name.end()), value)));	    //attach it to the parent node
 	}
     #endif
 }
 
 //Create a subarray
 void JSONWorker::DoArray(const internalJSONNode * parent, const json_string & value_t) json_nothrow {
-    //This takes an array and creates nodes out of them
-    JSON_ASSERT(!value_t.empty(), JSON_TEXT("DoArray is empty"));
-    JSON_ASSERT_SAFE(value_t[0] == JSON_TEXT('['), JSON_TEXT("DoArray is not an array"), parent -> Nullify(); return;);
-    if (json_unlikely(value_t.length() <= 2)) return;  // just a [] (blank array)
-
-    #ifdef JSON_SAFE
-	   json_string newValue;  //share this so it has a reserved buffer
-    #endif
-    size_t starting = 1;  //ignore the [
-
-    //Not sure what's in the array, so we have to use commas
-    for(size_t ending = FindNextRelevant(JSON_TEXT(','), value_t, 1);
-	   ending != json_string::npos;
-	   ending = FindNextRelevant(JSON_TEXT(','), value_t, starting)){
-
-	   #ifdef JSON_SAFE
-		  newValue.assign(value_t.begin() + starting, value_t.begin() + ending);
-		  JSON_ASSERT_SAFE(FindNextRelevant(JSON_TEXT(':'), newValue, 0) == json_string::npos, JSON_TEXT("Key/Value pairs are not allowed in arrays"), parent -> Nullify(); return;);
-		  NewNode(parent, json_global(EMPTY_JSON_STRING), newValue, true);
-	   #else
-		  NewNode(parent, json_global(EMPTY_JSON_STRING), json_string(value_t.begin() + starting, value_t.begin() + ending), true);
-	   #endif
-		  starting = ending + 1;
-    }
-    //since the last one will not find the comma, we have to add it here, but ignore the final ]
-
-    #ifdef JSON_SAFE
-	   newValue.assign(value_t.begin() + starting, value_t.end() - 1);
-	   JSON_ASSERT_SAFE(FindNextRelevant(JSON_TEXT(':'), newValue, 0) == json_string::npos, JSON_TEXT("Key/Value pairs are not allowed in arrays"), parent -> Nullify(); return;);
-	   NewNode(parent, json_global(EMPTY_JSON_STRING), newValue, true);
-    #else
-	   NewNode(parent, json_global(EMPTY_JSON_STRING), json_string(value_t.begin() + starting, value_t.end() - 1), true);
-    #endif
+	//This takes an array and creates nodes out of them
+	JSON_ASSERT(!value_t.empty(), JSON_TEXT("DoArray is empty"));
+	JSON_ASSERT_SAFE(value_t[0] == JSON_TEXT('['), JSON_TEXT("DoArray is not an array"), parent -> Nullify(); return;);
+	if (json_unlikely(value_t.length() <= 2)) return;  // just a [] (blank array)
+	
+	#ifdef JSON_SAFE
+		json_string newValue;  //share this so it has a reserved buffer
+	#endif
+	size_t starting = 1;  //ignore the [
+	
+	//Not sure what's in the array, so we have to use commas
+	for(size_t ending = FIND_NEXT_RELEVANT(JSON_TEXT(','), value_t, 1);
+		ending != json_string::npos;
+		ending = FIND_NEXT_RELEVANT(JSON_TEXT(','), value_t, starting)){
+		
+		#ifdef JSON_SAFE
+			newValue.assign(value_t.begin() + starting, value_t.begin() + ending);
+			JSON_ASSERT_SAFE(FIND_NEXT_RELEVANT(JSON_TEXT(':'), newValue, 0) == json_string::npos, JSON_TEXT("Key/Value pairs are not allowed in arrays"), parent -> Nullify(); return;);
+			NewNode(parent, json_global(EMPTY_JSON_STRING), newValue, true);
+		#else
+			NewNode(parent, json_global(EMPTY_JSON_STRING), json_string(value_t.begin() + starting, value_t.begin() + ending), true);
+		#endif
+		starting = ending + 1;
+	}
+	//since the last one will not find the comma, we have to add it here, but ignore the final ]
+	
+	#ifdef JSON_SAFE
+		newValue.assign(value_t.begin() + starting, value_t.end() - 1);
+		JSON_ASSERT_SAFE(FIND_NEXT_RELEVANT(JSON_TEXT(':'), newValue, 0) == json_string::npos, JSON_TEXT("Key/Value pairs are not allowed in arrays"), parent -> Nullify(); return;);
+		NewNode(parent, json_global(EMPTY_JSON_STRING), newValue, true);
+	#else
+		NewNode(parent, json_global(EMPTY_JSON_STRING), json_string(value_t.begin() + starting, value_t.end() - 1), true);
+	#endif
 }
 
 
 //Create all child nodes
-void JSONWorker::DoNode(const internalJSONNode * parent, const json_string & value_t) json_nothrow {
-    //This take a node and creates its members and such
-    JSON_ASSERT(!value_t.empty(), JSON_TEXT("DoNode is empty"));
-    JSON_ASSERT_SAFE(value_t[0] == JSON_TEXT('{'), JSON_TEXT("DoNode is not an node"), parent -> Nullify(); return;);
-    if (json_unlikely(value_t.length() <= 2)) return;  // just a {} (blank node)
-
-    size_t name_ending = FindNextRelevant(JSON_TEXT(':'), value_t, 1);  //find where the name ends
-    JSON_ASSERT_SAFE(name_ending != json_string::npos, JSON_TEXT("Missing :"), parent -> Nullify(); return;);
-    json_string name(value_t.begin() + 1, value_t.begin() + name_ending - 1);	  //pull the name out
-    for (size_t value_ending = FindNextRelevant(JSON_TEXT(','), value_t, name_ending),  //find the end of the value
-	    name_starting = 1;  //ignore the {
-	    value_ending != json_string::npos;
-	    value_ending = FindNextRelevant(JSON_TEXT(','), value_t, name_ending)){
-
-	   NewNode(parent, name, json_string(value_t.begin() + name_ending + 1, value_t.begin() + value_ending), false);
-	   name_starting = value_ending + 1;
-	   name_ending = FindNextRelevant(JSON_TEXT(':'), value_t, name_starting);
-	   JSON_ASSERT_SAFE(name_ending != json_string::npos, JSON_TEXT("Missing :"), parent -> Nullify(); return;);
-	   name.assign(value_t.begin() + name_starting, value_t.begin() + name_ending - 1);
-    }
-    //since the last one will not find the comma, we have to add it here
-    NewNode(parent, name, json_string(value_t.begin() + name_ending + 1, value_t.end() - 1), false);
+void JSONWorker::DoNode(const internalJSONNode * parent, const json_string & value_t) json_nothrow {	
+	//This take a node and creates its members and such
+	JSON_ASSERT(!value_t.empty(), JSON_TEXT("DoNode is empty"));
+	JSON_ASSERT_SAFE(value_t[0] == JSON_TEXT('{'), JSON_TEXT("DoNode is not an node"), parent -> Nullify(); return;);
+	if (json_unlikely(value_t.length() <= 2)) return;  // just a {} (blank node)
+	
+	size_t name_ending = FIND_NEXT_RELEVANT(JSON_TEXT(':'), value_t, 1);  //find where the name ends
+	JSON_ASSERT_SAFE(name_ending != json_string::npos, JSON_TEXT("Missing :"), parent -> Nullify(); return;);
+	json_string name(value_t.begin() + 1, value_t.begin() + name_ending - 1);	  //pull the name out
+	for (size_t value_ending = FIND_NEXT_RELEVANT(JSON_TEXT(','), value_t, name_ending),  //find the end of the value
+		 name_starting = 1;  //ignore the {
+		 value_ending != json_string::npos;
+		 value_ending = FIND_NEXT_RELEVANT(JSON_TEXT(','), value_t, name_ending)){
+		
+		NewNode(parent, name, json_string(value_t.begin() + name_ending + 1, value_t.begin() + value_ending), false);
+		name_starting = value_ending + 1;
+		name_ending = FIND_NEXT_RELEVANT(JSON_TEXT(':'), value_t, name_starting);
+		JSON_ASSERT_SAFE(name_ending != json_string::npos, JSON_TEXT("Missing :"), parent -> Nullify(); return;);
+		name.assign(value_t.begin() + name_starting, value_t.begin() + name_ending - 1);
+	}
+	//since the last one will not find the comma, we have to add it here
+	NewNode(parent, name, json_string(value_t.begin() + name_ending + 1, value_t.end() - 1), false);
 }
 #endif
